//
// PS2PC: An Intel x86 emulator for MIPS R5900 (Sony PS2).
// Coded by Mike Tedder / breakpoint - bpoint@parodius.com
// http://ps2pc.parodius.com
//
// This project is released as 'open source'.  You may make any modifications
// or adjustments to the code and submit a patch to be applied to the project.
// See the web page for more details.
//
//-----------------------------------------------------------------------

#include "types.h"
#include "x86.h"
#include "x86arith.h"
#include "r59basic.h"


//=====================
//        ADD
//=====================

int x86ADD_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADD_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADD_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADD_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADD_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADD_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


//=====================
//        ADC
//=====================

int x86ADC_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADC_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADC_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADC_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADC_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86ADC_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


//=====================
//        SUB
//=====================

int x86SUB_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SUB_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SUB_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SUB_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SUB_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SUB_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


//=====================
//        SBB
//=====================

int x86SBB_EbGb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SBB_EvGv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SBB_GbEb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SBB_GvEv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SBB_ALIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86SBB_eAXIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


//=====================
//        INC
//=====================

int x86INC_eAX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT0, rT0, 1);						// EAX += 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT0, 1);				// S0   = EAX + 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT0, rT0, rS0);				// EAX &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT0, rT0, rS1);				// EAX |= S1
		*skip += 5;
	}

	return 1;
}


int x86INC_eCX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT2, rT2, 1);						// ECX += 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT2, 1);				// S0   = ECX + 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT2, rT2, rS0);				// ECX &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT2, rT2, rS1);				// ECX |= S1
		*skip += 5;
	}

	return 1;
}


int x86INC_eDX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT3, rT3, 1);						// EDX += 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT3, 1);				// S0   = EDX + 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT3, rT3, rS0);				// EDX &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT3, rT3, rS1);				// EDX |= S1
		*skip += 5;
	}

	return 1;
}


int x86INC_eBX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT1, rT1, 1);						// EBX += 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT1, 1);				// S0   = EBX + 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT1, rT1, rS0);				// EBX &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT1, rT1, rS1);				// EBX |= S1
		*skip += 5;
	}

	return 1;
}


int x86INC_eSP(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT7, rT7, 1);						// ESP += 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT7, 1);				// S0   = ESP + 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT7, rT7, rS0);				// ESP &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT7, rT7, rS1);				// ESP |= S1
		*skip += 5;
	}

	return 1;
}


int x86INC_eBP(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT6, rT6, 1);						// EBP += 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT6, 1);				// S0   = EBP + 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT6, rT6, rS0);				// EBP &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT6, rT6, rS1);				// EBP |= S1
		*skip += 5;
	}

	return 1;
}


int x86INC_eSI(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT4, rT4, 1);						// ESI += 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT4, 1);				// S0   = ESI + 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT4, rT4, rS0);				// ESI &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT4, rT4, rS1);				// ESI |= S1
		*skip += 5;
	}

	return 1;
}


int x86INC_eDI(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT5, rT5, 1);						// EDI += 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT5, 1);				// S0   = EDI + 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT5, rT5, rS0);				// EDI &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT5, rT5, rS1);				// EDI |= S1
		*skip += 5;
	}

	return 1;
}


//=====================
//        DEC
//=====================

int x86DEC_eAX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT0, rT0, 0xFFFF);				// EAX -= 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT0, 0xFFFF);			// S0   = EAX - 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT0, rT0, rS0);				// EAX &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT0, rT0, rS1);				// EAX |= S1
		*skip += 5;
	}

	return 1;
}


int x86DEC_eCX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT2, rT2, 0xFFFF);				// ECX -= 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT2, 0xFFFF);			// S0   = ECX - 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT2, rT2, rS0);				// ECX &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT2, rT2, rS1);				// ECX |= S1
		*skip += 5;
	}

	return 1;
}


int x86DEC_eDX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT3, rT3, 0xFFFF);				// EDX -= 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT3, 0xFFFF);			// S0   = EDX - 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT3, rT3, rS0);				// EDX &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT3, rT3, rS1);				// EDX |= S1
		*skip += 5;
	}

	return 1;
}


int x86DEC_eBX(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT1, rT1, 0xFFFF);				// EBX -= 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT1, 0xFFFF);			// S0   = EBX - 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT1, rT1, rS0);				// EBX &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT1, rT1, rS1);				// EBX |= S1
		*skip += 5;
	}

	return 1;
}


int x86DEC_eSP(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT7, rT7, 0xFFFF);				// ESP -= 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT7, 0xFFFF);			// S0   = ESP - 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT7, rT7, rS0);				// ESP &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT7, rT7, rS1);				// ESP |= S1
		*skip += 5;
	}

	return 1;
}


int x86DEC_eBP(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT6, rT6, 0xFFFF);				// EBP -= 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT6, 0xFFFF);			// S0   = EBP - 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT6, rT6, rS0);				// EBP &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT6, rT6, rS1);				// EBP |= S1
		*skip += 5;
	}

	return 1;
}


int x86DEC_eSI(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT4, rT4, 0xFFFF);				// ESI -= 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT4, 0xFFFF);			// S0   = ESI - 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT4, rT4, rS0);				// ESI &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT4, rT4, rS1);				// ESI |= S1
		*skip += 5;
	}

	return 1;
}


int x86DEC_eDI(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	if (in32BitMode)
	{
		// Encode it...
		*out = r5900ADDIU(rT5, rT5, 0xFFFF);				// EDI -= 1
		*skip += 1;
	}
	else
	{
		* out      = r5900ADDIU(rS0, rT5, 0xFFFF);			// S0   = EDI - 1
		*(out + 1) = r5900ANDI (rS1, rS0, 0xFFFF);			// S1   = S0 & 0x0000FFFF      (only keep lower 16 bits!)
		*(out + 2) = r5900LUI  (rS0, 0xFFFF);				// S0   = 0xFFFF0000
		*(out + 3) = r5900AND  (rT5, rT5, rS0);				// EDI &= 0xFFFF0000
		*(out + 4) = r5900OR   (rT5, rT5, rS1);				// EDI |= S1
		*skip += 5;
	}

	return 1;
}


//=====================
//        IMUL
//=====================

int x86IMUL_GvEvIv(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


int x86IMUL_GvEvIb(uchar *in, ulong *out, bool *isBranch, ulong *skip)
{
	return 0;
}


